;; Created by Lee Ambrosius (lee_ambrosius@hyperpics.com), HyperPics all rights reserved.

;; Visit HyperPics on the web at: http://www.hyperpics.com

;; Date: 05/09/00



;; Version 1.1

;; BLKX is a complete Block Export program to a text file.

;; The text file is then automatically opened for viewing and saving

;; to a different file name.  Default file that is created everytime

;; is BLKX.txt and is placed in the working directory of AutoCAD.


;; Revised: 05/01/03
;; Version 1.2
;; Cleaned up the prompt to work with the right click of 2000+ and if the user
;; presses enter at the prompt to use Yes as default.

;; Date: 12/21/03
;; Version 1.3

;; Added support to print to Command Line or External file.


(defun c:BLKX ()



  ;; Grabs the value from the designated location in the DXF code

  ;; ie. (dxf 2 <ENTITY DATA>)

  (defun dxf (code elist)

    (cdr (assoc code elist))

  )



  ;; Opening prompt of application

  (prompt "\nStarting up Block Export...")



  ;; Users decides whether or not to have *, AME_ or AVE_ blocks added to the list

  (initget "Yes No")

  (setq KWORD_EXCLUDE (getkword "\nExclude *,AME_ and AVE_ blocks [Yes/No]<Y>: "))


  ;; User just pressed Enter to accept"Y" so set it to "Yes"
  (if (= KWORD_EXCLUDE nil)(setq KWORD_EXCLUDE "Yes"))

  ;; Users decides whether or not to have *, AME_ or AVE_ blocks added to the list

  (initget "Command File")

  (setq KWORD_EXPORT (getkword "\nExport to [Command line/File] <Command>: "))


  ;; User just pressed Enter to accept "Command" so set it to "Command"
  (if (= KWORD_EXPORT nil)(setq KWORD_EXPORT "Command"))


  ;; Notification that exporting is going to start

  (prompt "\nExporting Block list...")



  ;; Grab the current value of highlight and then set it to 0

  (setq SYS_HIGHLIGHT (getvar "highlight"))

  (setvar "highlight" 0)



  ;; Grab very first Layer entity in Layer Table

  (setq ED (tblnext "BLOCK" T))



  ;; Grab the first Layer name from table entity 

  (setq BLKNAM (dxf 2 ED))


  (if (= KWORD_EXPORT "File")
    (progn

      ;; Open the BLKX.TXT file for write

      (setq FP (open "blkx.txt" "w"))



      ;; Write the file header information

      (write-line "     Block name                  Drawing Count" FP)

      (write-line "------------------------------ -----------------" FP)

    )
    (progn
      ;; Call up the Text Screen
      (textscr)
      ;; Write the file header information

      (princ "\n\n     Block name                  Drawing Count")

      (princ "\n------------------------------ -----------------")
    )
  )


  ;; Set counter to 0

  (setq count 1)



  ;; Keep looping while there is an entity in the layer table

  (while (/= BLKNAM nil)



    ;; Create a selection set out of all the blocks in the drawing 

    (setq SS_BLKNAM (ssget "X" (list (cons 0 "INSERT")(cons 2 BLKNAM))))

    

    ;; Get the count of the number of blocks in the selection set if any

    (if (/= SS_BLKNAM nil)

      (setq BLK_CNT (itoa (sslength SS_BLKNAM)))

      (setq BLK_CNT "0")

    )



    ;; If the Layer name is less than 32 characters then add some extra spaces in the value

    (while (< (strlen BLKNAM) 33)

      (setq BLKNAM (strcat BLKNAM " "))

    )



    ;; Exclude blocks that start with "*", "AVE_" or "AME_" 

    (if (and (/= (substr BLKNAM 1 1) "*")(/= (substr BLKNAM 1 4) "AVE_")(/= (substr BLKNAM 1 4) "AME_"))


      ;; Write block information out to the BLKX.TXT file
      (if (= KWORD_EXPORT "File")

        (write-line (strcat BLKNAM BLK_CNT) FP)
        (princ (strcat "\n" BLKNAM BLK_CNT))
      )

    );; end if



    ;; Exclude blocks that start with "*", "AVE_" or "AME_" 

    (if (and (or (= (substr BLKNAM 1 1) "*")(= (substr BLKNAM 1 4) "AVE_")(= (substr BLKNAM 1 4) "AME_"))(= KWORD_EXCLUDE "No"))



      ;; Write block information out to the BLKX.TXT file
      (if (= KWORD_EXPORT "File")

        (write-line (strcat BLKNAM BLK_CNT) FP)
        (princ (strcat "\n" BLKNAM BLK_CNT))

      )

    );; end if



    ;; Grab the next Block entity in Block Table

    (setq ED (tblnext "BLOCK"))



    ;; Display progress of export based on blocks
    (if (= KWORD_EXPORT "File")
      (progn

        (prompt (itoa count))
        (princ)
        (repeat (strlen (itoa count))(prompt (chr 8)))
      )
    )



    ;; Add one to the counter

    (setq count (1+ count))



    ;; Grab the Block name from table entity

    (setq BLKNAM (dxf 2 ED))



  )


  (if (= KWORD_EXPORT "File")
    (prompt " blocks exported.")
  )

  ;; Close the file BLKX.TXT for write
  (if (= KWORD_EXPORT "File")
    (progn

      (close FP)


      ;; Launch WordPad with the BLKX.TXT file

      (startapp "write.exe" "blkx.txt")
    )

  )


 (princ)

)



(defun c:BLKX? ()

  (textscr)

  (prompt "\n")

  (prompt "\nCreated by Lee Ambrosius, HyperPics all rights reserved.")

  (prompt "\nVisit HyperPics on the web at: www.hyperpics.com")

  (prompt "\nDate: 5/09/00 Revised: 12/21/03")

  (prompt "\n")

  (prompt "\nVersion 1.3")

  (prompt "\n")

  (prompt "\nBLKX is a complete Block Export program to a text file.")

  (prompt "\nThe text file is then automatically opened for viewing and saving")

  (prompt "\nto a different file name.  Default file that is created everytime")

  (prompt "\nis BLKX.txt and is placed in the working directory of AutoCAD.")

 (prompt "\n")

 (princ)

)



(prompt "\n")

(prompt "\nCopyright(c) 2000,2003 Block Export tool.")

(prompt "\nAll rights reserved by HyperPics, www.hyperpics.com .")

(prompt "\nType BLKX to create a layer list or BLKX? for Help.")

